@@ -1,11 +1,12 @@
+# -*- coding: utf-8 -*-
+
 import struct
 
-from pymysql.util import byte2int, int2byte
-from .constants.BINLOG import *
-from .event import *
-from .row_event import *
+from pymysql.util import byte2int
 
-#Constants from PyMYSQL source code
+from pymysqlreplication import constants, event, row_event
+
+# Constants from PyMYSQL source code
 NULL_COLUMN = 251
 UNSIGNED_CHAR_COLUMN = 251
 UNSIGNED_SHORT_COLUMN = 252
@@ -16,6 +17,7 @@ UNSIGNED_SHORT_LENGTH = 2
 UNSIGNED_INT24_LENGTH = 3
 UNSIGNED_INT64_LENGTH = 8
 
+
 class BinLogPacketWrapper(object):
     """
     Bin Log Packet Wrapper. It uses an existing packet object, and wraps
@@ -24,26 +26,40 @@ class BinLogPacketWrapper(object):
     """
 
     __event_map = {
-        QUERY_EVENT: QueryEvent,
-        UPDATE_ROWS_EVENT_V1: UpdateRowsEvent,
-        WRITE_ROWS_EVENT_V1: WriteRowsEvent,
-        DELETE_ROWS_EVENT_V1: DeleteRowsEvent,
-        UPDATE_ROWS_EVENT: UpdateRowsEvent,
-        WRITE_ROWS_EVENT: WriteRowsEvent,
-        DELETE_ROWS_EVENT: DeleteRowsEvent,
-        TABLE_MAP_EVENT: TableMapEvent,
-        ROTATE_EVENT: RotateEvent,
-        FORMAT_DESCRIPTION_EVENT: FormatDescriptionEvent,
-        XID_EVENT: XidEvent
+        # event
+        constants.QUERY_EVENT: event.QueryEvent,
+        constants.ROTATE_EVENT: event.RotateEvent,
+        constants.FORMAT_DESCRIPTION_EVENT: event.FormatDescriptionEvent,
+        constants.XID_EVENT: event.XidEvent,
+        constants.INTVAR_EVENT: event.NotImplementedEvent,
+        constants.GTID_LOG_EVENT: event.NotImplementedEvent,
+        constants.STOP_EVENT: event.NotImplementedEvent,
+        # row_event
+        constants.UPDATE_ROWS_EVENT_V1: row_event.UpdateRowsEvent,
+        constants.WRITE_ROWS_EVENT_V1: row_event.WriteRowsEvent,
+        constants.DELETE_ROWS_EVENT_V1: row_event.DeleteRowsEvent,
+        constants.UPDATE_ROWS_EVENT_V2: row_event.UpdateRowsEvent,
+        constants.WRITE_ROWS_EVENT_V2: row_event.WriteRowsEvent,
+        constants.DELETE_ROWS_EVENT_V2: row_event.DeleteRowsEvent,
+        constants.TABLE_MAP_EVENT: row_event.TableMapEvent,
+        #5.6 GTID enabled replication events
+        constants.INTVAR_EVENT: event.NotImplementedEvent,
+        constants.GTID_LOG_EVENT: event.NotImplementedEvent,
+        constants.ANONYMOUS_GTID_LOG_EVENT: event.NotImplementedEvent,
+        constants.PREVIOUS_GTIDS_LOG_EVENT: event.NotImplementedEvent
+
     }
 
-    def __init__(self, from_packet, table_map, ctl_connection):
+    def __init__(self, from_packet, table_map, ctl_connection, use_checksum):
         if not from_packet.is_ok_packet():
-            raise ValueError('Cannot create ' + str(self.__class__.__name__)
-                + ' object from invalid packet type')
+            raise ValueError(
+                "Cannot create %s object from invalid packet type" %
+                self.__class__.__name__)
 
-        self.read_bytes = 0 #-1 because we ignore the ok byte
-        self.__data_buffer = b'' #Used when we want to override a value in the data buffer
+        # -1 because we ignore the ok byte
+        self.read_bytes = 0
+        # Used when we want to override a value in the data buffer
+        self.__data_buffer = b''
 
         # Ok Value
         self.packet = from_packet
@@ -57,15 +73,24 @@ class BinLogPacketWrapper(object):
         self.event_size = struct.unpack('<I', self.packet.read(4))[0]
         # position of the next event
         self.log_pos = struct.unpack('<I', self.packet.read(4))[0]
-        self.flags = self.flags = struct.unpack('<H', self.packet.read(2))[0]
+        self.flags = struct.unpack('<H', self.packet.read(2))[0]
+
+        # MySQL 5.6 and more if binlog-checksum = CRC32
+        if use_checksum:
+            event_size_without_header = self.event_size - 23
+        else:
+            event_size_without_header = self.event_size - 19
 
 
-        event_size_without_header = self.event_size - 19
         try:
             event_class = self.__event_map[self.event_type]
         except KeyError:
-            raise NotImplementedError("Unknown MySQL bin log event type: " + hex(self.event_type) + " (" + str(self.event_type) + ")")
-        self.event = event_class(self, event_size_without_header, table_map, ctl_connection)
+            raise NotImplementedError(
+                "Unknown MySQL bin log event type: %s (%s)" %
+                (hex(self.event_type), self.event_type))
+
+        self.event = event_class(self, event_size_without_header, table_map,
+                                 ctl_connection)
 
     def read(self, size):
         size = int(size)
@@ -107,15 +132,15 @@ class BinLogPacketWrapper(object):
         """
         c = byte2int(self.read(1))
         if c == NULL_COLUMN:
-          return None
+            return None
         if c < UNSIGNED_CHAR_COLUMN:
-          return c
+            return c
         elif c == UNSIGNED_SHORT_COLUMN:
             return self.unpack_uint16(self.read(UNSIGNED_SHORT_LENGTH))
         elif c == UNSIGNED_INT24_COLUMN:
-          return self.unpack_int24(self.read(UNSIGNED_INT24_LENGTH))
+            return self.unpack_int24(self.read(UNSIGNED_INT24_LENGTH))
         elif c == UNSIGNED_INT64_COLUMN:
-          return self.unpack_int64(self.read(UNSIGNED_INT64_LENGTH))
+            return self.unpack_int64(self.read(UNSIGNED_INT64_LENGTH))
 
     def read_length_coded_string(self):
         """Read a 'Length Coded String' from the data buffer.
@@ -135,8 +160,8 @@ class BinLogPacketWrapper(object):
         if hasattr(self.packet, key):
             return getattr(self.packet, key)
 
-        raise AttributeError(str(self.__class__)
-            + " instance has no attribute '" + key + "'")
+        raise AttributeError("%s instance has no attribute '%s'" %
+                             (self.__class__, key))
 
     def read_int_be_by_size(self, size):
         '''Read a big endian integer values based on byte number'''
@@ -173,27 +198,25 @@ class BinLogPacketWrapper(object):
             return self.read_uint64()
 
     def read_length_coded_pascal_string(self, size):
-        '''Read a string with length coded using pascal style. The string start by the size of the string'''
+        """Read a string with length coded using pascal style.
+        The string start by the size of the string
+        """
         length = self.read_uint_by_size(size)
         return self.read(length)
 
     def read_int24(self):
         a, b, c = struct.unpack("BBB", self.read(3))
-        if a & 128:
-            return a + (b << 8) + (c << 16)
-        else:
-            return (a + (b << 8) + (c << 16)) * -1
+        res = a | (b << 8) | (c << 16)
+        if res >= 0x800000:
+            res -= 0x1000000
+        return res
 
     def read_int24_be(self):
         a, b, c = struct.unpack('BBB', self.read(3))
-        if a & 128 == 0:
-            return (a << 16) | (b << 8) | c
-        else:
-            return (-1 << 24) | (a << 16) | (b << 8) | c
-
-    def read_uint24(self, unsigned = False):
-        a, b, c = struct.unpack("BBB", self.read(3))
-        return a + (b << 8) + (c << 16)
+        res = (a << 16) | (b << 8) | c
+        if res >= 0x800000:
+            res -= 0x1000000
+        return res
 
     def read_uint8(self):
         return struct.unpack('<B', self.read(1))[0]
@@ -202,27 +225,27 @@ class BinLogPacketWrapper(object):
         return struct.unpack('<H', self.read(2))[0]
 
     def read_uint24(self):
-      a, b, c = struct.unpack("<BBB", self.read(3))
-      return a + (b << 8) + (c << 16)
+        a, b, c = struct.unpack("<BBB", self.read(3))
+        return a + (b << 8) + (c << 16)
 
     def read_uint32(self):
         return struct.unpack('<I', self.read(4))[0]
 
     def read_uint40(self):
-      a, b = struct.unpack("<BI", self.read(5))
-      return a + (b << 8)
+        a, b = struct.unpack("<BI", self.read(5))
+        return a + (b << 8)
 
     def read_int40_be(self):
-      a, b = struct.unpack(">IB", self.read(5))
-      return b + (a << 8)
+        a, b = struct.unpack(">IB", self.read(5))
+        return b + (a << 8)
 
     def read_uint48(self):
-      a, b, c = struct.unpack("<HHH", self.read(6))
-      return a + (b << 16) + (c << 32)
+        a, b, c = struct.unpack("<HHH", self.read(6))
+        return a + (b << 16) + (c << 32)
 
     def read_uint56(self):
-      a, b, c = struct.unpack("<BHI", self.read(7))
-      return a + (b << 8) + (c << 24)
+        a, b, c = struct.unpack("<BHI", self.read(7))
+        return a + (b << 8) + (c << 24)
 
     def read_uint64(self):
         return struct.unpack('<Q', self.read(8))[0]
@@ -231,18 +254,21 @@ class BinLogPacketWrapper(object):
         return struct.unpack('<q', self.read(8))[0]
 
     def unpack_uint16(self, n):
-      return struct.unpack('<H', n[0:2])[0]
+        return struct.unpack('<H', n[0:2])[0]
 
     def unpack_int24(self, n):
         try:
-            return struct.unpack('B',n[0])[0] + (struct.unpack('B', n[1])[0] << 8) +\
-                (struct.unpack('B',n[2])[0] << 16)
+            return struct.unpack('B', n[0])[0] \
+                + (struct.unpack('B', n[1])[0] << 8) \
+                + (struct.unpack('B', n[2])[0] << 16)
         except TypeError:
             return n[0] + (n[1] << 8) + (n[2] << 16)
 
     def unpack_int32(self, n):
         try:
-            return struct.unpack('B',n[0])[0] + (struct.unpack('B', n[1])[0] << 8) +\
-                (struct.unpack('B',n[2])[0] << 16) + (struct.unpack('B', n[3])[0] << 24)
+            return struct.unpack('B', n[0])[0] \
+                + (struct.unpack('B', n[1])[0] << 8) \
+                + (struct.unpack('B', n[2])[0] << 16) \
+                + (struct.unpack('B', n[3])[0] << 24)
         except TypeError:
             return n[0] + (n[1] << 8) + (n[2] << 16) + (n[3] << 24)
