@@ -1,13 +1,17 @@
+# -*- coding: utf-8 -*-
+
 import pymysql
 import unittest
 import copy
 from pymysqlreplication import BinLogStreamReader
 
+
 class PyMySQLReplicationTestCase(unittest.TestCase):
-    '''Test the module. Be carefull it will reset your MySQL server'''
-    database =  {"host":"localhost",
-        "user":"root",
-        "passwd":"",
+    """Test the module. Be carefull it will reset your MySQL server"""
+    database = {
+        "host": "localhost",
+        "user": "root",
+        "passwd": "",
         "use_unicode": True,
         "charset": "utf8",
         "db": "pymysqlreplication_test"
@@ -27,8 +31,9 @@ class PyMySQLReplicationTestCase(unittest.TestCase):
         self.isMySQL56AndMore()
 
     def getMySQLVersion(self):
-        '''Return the MySQL version of the server
-        If version is 5.6.10-log the result is 5.6.10'''
+        """Return the MySQL version of the server
+        If version is 5.6.10-log the result is 5.6.10
+        """
         return self.execute("SELECT VERSION()").fetchone()[0].split('-')[0]
 
     def isMySQL56AndMore(self):
@@ -57,5 +62,4 @@ class PyMySQLReplicationTestCase(unittest.TestCase):
         self.execute("RESET MASTER")
         if self.stream is not None:
             self.stream.close()
-        self.stream = BinLogStreamReader(connection_settings = self.database)
-
+        self.stream = BinLogStreamReader(connection_settings=self.database)
